<?php
/*
Plugin Name: Admin Toolbox
Plugin URI: http://richardlerma.com/r1cm
Description: Manage an array of administrative options improving user control and resource management.
Author: R1CM
Version: 5.0.1
Text Domain: r1cm
Author URI: http://richardlerma.com/r1cm
License: GPL2
*/

global $atb_version; $atb_version='5.0.1';
if(!defined('ABSPATH')) exit;

function atb_startSession() {if(!session_id()) session_start();}
add_action('init','atb_startSession',1);

function atb_error() {file_put_contents(dirname(__file__).'/error_activation.txt', ob_get_contents());}
if(defined('WP_DEBUG') && true===WP_DEBUG) add_action('activated_plugin','atb_error');

function atb_adminMenu() {
  if(current_user_can('manage_options')) {
    add_options_page('Admin Toolbox','Admin Toolbox','manage_options','admin-toolbox','atb_admin');
  }
}
add_action('admin_menu','atb_adminMenu');

function atb_admin() {
  global $wpdb;
  global $atb_version;
  include_once('atb_admin.php');
  wp_enqueue_style('atb_style',plugins_url('assets/atb_min.css?v='.$atb_version,__FILE__));
}
add_shortcode('atb_admin','atb_admin');

function atb_activate($upgrade) {
  global $wpdb;
  global $atb_version;
  require_once(ABSPATH.'wp-admin/includes/upgrade.php');
  update_option('atb_db_version',$atb_version,'no');
  
  $sql="
    CREATE TABLE {$wpdb->prefix}atb_options
    (option_id INT NOT NULL AUTO_INCREMENT
    ,option_name VARCHAR(191) NULL
    ,option_value VARCHAR(500)
    ,PRIMARY KEY  (option_id));";
  dbDelta($sql);

  $sql="
    CREATE TABLE {$wpdb->prefix}atb_pagehits
    (hit_id INT NOT NULL AUTO_INCREMENT
    ,date TIMESTAMP DEFAULT CURRENT_TIMESTAMP
    ,ip VARCHAR(191) NULL
    ,userid INT
    ,url VARCHAR(500)
    ,referrer VARCHAR(500)
    ,PRIMARY KEY  (hit_id)
    ,KEY date (date));";
  dbDelta($sql);

  if(function_exists('atb_pro_ping'))atb_pro_ping(2);
  ?><script type'text/javascript'>window.location.href='<?php echo get_admin_url(null,'admin.php?page=admin-toolbox');?>&updated=1';</script><?php 
}
register_activation_hook(__FILE__,'atb_activate');

if(atb_is_path('/wp-admin/plugins.php') && atb_is_path('plugin=admin-toolbox')) { ?><style type='text/css'>div.error{display:none!important}</style><?php }

function atb_admin_notice() {
  if(!atb_is_path('page=admin-toolbox')){
    require_once(ABSPATH."wp-includes/pluggable.php");
    if(current_user_can('manage_options')) {
      $settings_url=get_admin_url(null,'admin.php?page=admin-toolbox');?>
      <div class="notice notice-success is-dismissible" style='margin:0;'>
        <p><?php _e("The <em>Admin Toolbox</em> plugin is active, but is not yet configured. Visit the <a href='$settings_url'>configuration page</a> to complete setup.",'Admin Toolbox');?>
      </div><?php
    }
  }
}

function atb_checkConfig() {
  global $wpdb;
  $get_config=$wpdb->get_results("SELECT 1 FROM {$wpdb->prefix}atb_options HAVING COUNT(*)>1;",OBJECT);
  if(!$get_config) add_action('admin_notices','atb_admin_notice');
}
add_action('admin_init','atb_checkConfig');

function atb_add_action_links($links) {
  $settings_url=get_admin_url(null,'admin.php?page=admin-toolbox');
  $support_url='http://richardlerma.com/r1cm/';
  $links[]='<a href="'.$support_url.'">Support</a>';
  array_push($links,'<a href="'.$settings_url.'">Settings</a>');
  return $links;
}
add_filter('plugin_action_links_'.plugin_basename(__FILE__),'atb_add_action_links');

function atb_uninstall() {
  global $wpdb;
  $get_config=$wpdb->get_results("SELECT 1 FROM {$wpdb->prefix}atb_options WHERE option_name='page_hit_keep' AND option_value>0;",OBJECT);
  if(!$get_config) {
    $wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}atb_options;");
    $wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}atb_pagehits;");
    delete_option('atb_db_version');
  }
}
register_uninstall_hook(__FILE__,'atb_uninstall');

function atb_admin_impact() {global $wpdb;include_once('atb_admin_impact.php');}
add_action('admin_init','atb_admin_impact');

function atb_login_impact() {global $wpdb;include_once('atb_login_impact.php');}
add_action('login_enqueue_scripts','atb_login_impact');

function atb_frontend_impact() {
  global $wpdb;
  include_once('atb_frontend_impact.php');
}
if(!is_admin()) add_action('init','atb_frontend_impact');


function atb_cui($userid=0) {
  global $current_user;
  include_once(ABSPATH .'wp-includes/pluggable.php'); // If called prior to pluggable loaded natively
  if(current_user_can('manage_options') && $userid>0) return $userid;
  if(is_user_logged_in()) {
    $current_user=wp_get_current_user();
    $userid=$current_user->ID;
  }
  return $userid;
}


function atb_cur($user_id=0) {
  $user_role='';
  include_once(ABSPATH .'wp-includes/pluggable.php'); // If called prior to pluggable loaded natively
  if(is_user_logged_in()) {
    $user_id=atb_cui($user_id);
    $user_info=get_userdata($user_id);
    $user_role=$user_info->roles;
    $_SESSION["user_role"]=$user_role;
  }
  return $user_role;
}

function atb_r ($q,$t=NULL) {
  include_once(ABSPATH .'wp-includes/pluggable.php'); // If called prior to pluggable loaded natively
  require_once(ABSPATH .'wp-includes/wp-db.php');
  global $wpdb;
  if(!$wpdb) $wpdb=new wpdb(DB_USER,DB_PASSWORD,DB_NAME,DB_HOST);
  $prf=$wpdb->prefix;
  $s=str_replace(' wp_',' '.$prf,$q);
  $s=str_replace($prf.str_replace('wp_','',$prf),$prf,$s);
  if(strpos($s,'DELETE')!==false || strpos($s,'INSERT')!==false) $r=$wpdb->query($s); else $r=$wpdb->get_results($s,OBJECT);
  if($t) {echo $wpdb->last_error."<br>";echo $s;}
  if($r) return $r;
}

function atb_is_path($pages) {
  $page_array=explode(',',$pages);
  $current_page=strtolower($_SERVER['REQUEST_URI']);
  foreach($page_array as $page) {
    if(strpos($current_page,strtolower($page))!==false) return true;
  }
  return false;
}


// PageHit Action
function atb_hit_page() {
  if(atb_is_path('toolbox&phv=1,admin-ajax.php,wp-cron.php')!==false) return;
  global $current_user;
  $userid=0;
  $ip=sanitize_text_field($_SERVER['REMOTE_ADDR']);
  $url=sanitize_text_field(substr($_SERVER['REQUEST_URI'],0,250));
  if(is_user_logged_in()) $userid=atb_cui();
  $page=get_transient('atb_'.$ip.':'.$userid);
  if($url!=$page) {
    if(isset($_SERVER['HTTP_REFERER'])) $referrer=sanitize_text_field(substr($_SERVER['HTTP_REFERER'],0,250)); else $referrer='';
    set_transient('atb_'.$ip.':'.$userid,$url,3600);
    atb_r("INSERT INTO wp_atb_pagehits(ip,userid,url,referrer) VALUES ('$ip','$userid','$url','$referrer');");
  }
}

// Max Image Size Err Msg
function atb_media_size_msg() { ?>
  <style type='text/css'>
    .upload-error-message::before{content:"\AOPTIMIZE THIS CONTENT FOR WEB:\A";white-space:pre;font-weight:bold}
    .upload-error-message::after{content:" Decrease the file size to less than <?php echo $_SESSION["limit_img_size_kb"];?>KB by compressing the file, changing the file format or reducing the dimensions. Properly formatted and <a href='https://tinypng.com/' target='_blank'>compressed</a> images can save many bytes of data, decrease page loads times, increase SEO ranking, and improve overall user experience."}
  </style><?php
}

// Limit Media Library File Sizes
function atb_check_media_limit() {
  if(!empty($_SESSION["limit_img_size_kb"])) if($_SESSION["limit_img_size_kb"]>0) return true;
  if(atb_is_path('/wp-admin') && !atb_is_path('/wp-admin/plugins.php')) {

    $user_roles=atb_cur();
    if(empty($user_roles)) $user_roles=array();
    $get_limit=atb_r("SELECT option_name,option_value FROM wp_atb_options WHERE option_name LIKE 'limit_img_size%';");

    $limit_img_size=0;
    $limit_img_size_kb=0;
    
    if($get_limit):
      foreach($get_limit as $row):
        if($row->option_name=='limit_img_size') $limit_img_size=$row->option_value;
        if($row->option_name=='limit_img_size_role') $limit_img_size_role=explode(",",$row->option_value);
        if($row->option_name=='limit_img_size_role_exclude') $limit_img_size_role_exclude=$row->option_value;
        if($row->option_name=='limit_img_size_kb') $limit_img_size_kb=$row->option_value;
      endforeach;
    endif;
    
    if($limit_img_size==1 && $limit_img_size_kb>0) {
      if(($limit_img_size_role_exclude==0 && (count(array_intersect($limit_img_size_role,$user_roles))>0 || in_array('*All',$limit_img_size_role,true)))
       || ($limit_img_size_role_exclude==1 && (count(array_intersect($limit_img_size_role,$user_roles))==0 || in_array('*All',$limit_img_size_role,true)))) {
          $_SESSION["limit_img_size_kb"]=$limit_img_size_kb;
          return true;
      }
    }
  }
  return false;
}

function atb_media_limit() {
  atb_check_media_limit();
  add_action('admin_head','atb_media_size_msg');
  $kb=$_SESSION["limit_img_size_kb"]*1000;
  return $kb;
}
if(atb_check_media_limit()) add_filter('upload_size_limit','atb_media_limit');



// Remove other plugins' login features if logged in
if(atb_cui()>0) remove_all_actions('login_init'); 


// Configure Login Page
function atb_login_init() {
  if(atb_is_path('action=logout')) return;
  if(atb_is_path('action=lostpassword') || atb_is_path('2fa=1')) {
    if(is_user_logged_in()) {
      remove_all_actions('login_init'); // Remove other plugins' features
      add_action('login_init','atb_prompt_token');
    }
  } else add_action('login_init','atb_login_redirect');
}
add_action('wp_loaded','atb_login_init',0); // Renamed Login Page (AIOWPS)
add_action('wp_loaded','atb_login_init',999); // Standard wp-login.php


// Add Token on Login
function atb_create_token() {
  atb_startSession();
  $newcode=uniqid();
  $_SESSION['atb_token']=strtoupper(substr($newcode,8,5));
  $_SESSION['atb_flag']=1;
}
add_action('wp_login','atb_create_token');


// Email Functions
function atb_html_mail() {return 'text/html';}
function atb_mail_from($email) {return get_bloginfo('admin_email');}
function atb_mail_name($name) {return get_bloginfo('name');}
function atb_email_token($target) {
  $user_info=get_userdata(atb_cui());
  $title=get_bloginfo('name');
  $to=$user_info->user_email;
  if(!isset($_SESSION['atb_target'])) $_SESSION['atb_target']=$target;
  if(atb_is_path('rsd=2')) atb_create_token(); // regen token
  if(atb_is_path('rsd=1')) $_SESSION['atb_flag']=1; // resend token
  if($_SESSION['atb_flag']==1) { // if token not yet sent
    if(strpos($to,'@')!==false) { // check for valid email
      $name=$user_info->first_name;
      require_once(ABSPATH.WPINC.'/pluggable.php');
      $subject=$title.' Authorization Code';
      $intro="Dear ".$name."<br><br>&nbsp;";
      $message="Your $title authorization code is: ".$_SESSION['atb_token'];
      if(function_exists('atb_text')) $sent=atb_text($user_info->ID,$message);
      if(!$sent) {
        add_filter('wp_mail_content_type','atb_html_mail');
        add_filter('wp_mail_from','atb_mail_from');
        add_filter('wp_mail_from_name','atb_mail_name');
        wp_mail($to,$subject,$intro.$message);
        $to=substr($to,0,3)."<span style='-webkit-filter:blur(3px);filter:blur(3px)'>81818</span>".substr($to,-6);
        $_SESSION['atb_prompt_mobile']=1;
        remove_filter('wp_mail_content_type','atb_html_mail');
      } else $to=substr($sent,0,3)."<span style='-webkit-filter:blur(3px);filter:blur(3px)'>8181818</span>".substr($sent,-2);
      $_SESSION['atb_flag']=2;
      $_SESSION['atb_sent_to']=$to;
    } else unset($_SESSION['atb_flag']);
  }
  wp_redirect(wp_login_url()."?action=lostpassword&2fa=1#token"); exit;
}


// Show Token Entry Form
function atb_prompt_token() {
  if(atb_is_path('2fa=1')) {
    if(!empty($_POST['token'])) $token=sanitize_text_field($_POST['token']); else $token=false;
    if(!isset($_SESSION['atb_flag'])) atb_login_redirect();
    if(($token && (strtolower($_SESSION['atb_token'])==strtolower($token)))) {
      $target=$_SESSION['atb_target'];
      if(isset($_SESSION['atb_prompt_mobile'])) if(strpos($target,'?')===false) $target.="?atb_mob=1"; else $target.="&atb_mob=1";
      unset($_SESSION['atb_prompt_mobile']);
      unset($_SESSION['atb_target']);
      unset($_SESSION['atb_flag']);
      unset($_SESSION['atb_token']);
      unset($_SESSION['atb_sent_to']);
      wp_redirect(atb_login_redirect($target)); exit;
    } else {

      if($token) $error_msg="<div id='login_error'><strong>ERROR</strong>: Invalid Code</div>"; else $error_msg=''; ?>
      
      <title>Two Factor Authentication</title>
      <body class='login'>
        <style type='text/css'>
          #login{display:none!important}
          #auth{display:table;margin:auto}
          #auth input{padding:1em}
          #auth .resend{float:right;text-align:right}
          #auth .resend a{text-decoration:none}
          #auth input[type="submit"]{background:none;border:1px solid #02a0d2}
          #auth input[type="submit"]:hover,#auth input[type="submit"]:active,#auth input[type="submit"]:focus{background:#02a0d2;color:#FFF}
        </style>

        <form id='auth' method='post' accept-charset='UTF-8 ISO-8859-1'>
          <p class='message'>Two Factor Authentication<br><br>
            Please enter the code sent to <span style='white-space:nowrap;font-style:italic'><?php echo $_SESSION['atb_sent_to'];?></span></p>
          <?php echo $error_msg;?>
          <input type='text' id='token' name='token' autocomplete='off' maxlength='6' placeholder='Authorization Code'>
          <input type='submit' value='Verify'>
          <div class='resend'>
          <a href="<?php echo admin_url();?>?rsd=1" 
            onclick="if(!confirm('If you have not received a code, check your spam folder and allow up to 5 minutes for delivery.\n\nAre you sure you want to resend the code?')) return false;">resend code</a><br>
          <a href="<?php echo admin_url();?>?rsd=2" 
            onclick="if(!confirm('A new code VOIDS previous codes sent to you.\n\nAre you sure you want to generate a new code?')) return false;">generate new code</a>
          </div>
        </form>
      </body><?php
    }
  }
}


// Redirect from Login Page
function atb_login_redirect($target='') {
  if(!empty($target)) {
    if(strpos($target,'http')===false) if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']!='off') $http='https://'; else $http='http://';
    return $http.$target;
  }

  if(is_user_logged_in()) {
    if(current_user_can('edit_posts')) if(!atb_is_path('wp-admin')) {wp_redirect(admin_url()); exit;} else return;
    else wp_redirect(site_url());
  }
}


// Compress PageHits
function atb_compress_hits() {
  $r=atb_r("DELETE pg
  FROM wp_atb_pagehits pg
  JOIN (
    SELECT MAX(hit_id) as hit_id
    FROM wp_atb_pagehits
    WHERE userid<1
    AND date < NOW()-INTERVAL 60 DAY
    GROUP BY LEFT(date,13),ip,userid,url,referrer
    HAVING COUNT(ip)>1
    ORDER BY hit_id ASC
    LIMIT 1000
  )mx ON mx.hit_id=pg.hit_id;");
  return $r;
}


// PageHit Summary
function atb_view_summary($atb_version_type) {
  if(isset($_SESSION["limit_img_size_kb"])) unset($_SESSION["limit_img_size_kb"]);
  if(!empty($_REQUEST['compress'])) $compress_result=atb_compress_hits();

  $mth_query="
    SELECT DATE_FORMAT(CONCAT(month,'-01'),'%b %y') as month
    ,month as month_id,visitors
    ,ROUND(visitors/CASE WHEN month=DATE_FORMAT(NOW(),'%Y-%m') THEN DAY(NOW()) ELSE 30 END,0) as day_avg
    FROM (
      SELECT LEFT(date,7) as month,COUNT(DISTINCT ip) as visitors
      FROM wp_atb_pagehits
      WHERE date>NOW()-INTERVAL 400 DAY
      GROUP BY LEFT(date,7)
    )a
    ORDER BY DATE_FORMAT(CONCAT(month,'-01'),'%y-%m');";
  $mth=atb_r($mth_query);

  $day_query="
    SELECT DATE_FORMAT(date,'%a<br>%D') as day
    ,date as date_id
    ,DATE_FORMAT(CONCAT(date,'-01'),'%b %y') as month,visitors
    ,visitors
    FROM (
      SELECT LEFT(date,10) as date,COUNT(DISTINCT ip) as visitors
      FROM wp_atb_pagehits
      WHERE date>NOW()-INTERVAL 400 DAY
      GROUP BY LEFT(date,10)
    )a
    ORDER BY date;";
  $day=atb_r($day_query);
  
  $max_visit_avg=0;
  $max_visit_tot=0;
  foreach($mth as $mth_item) if($mth_item->day_avg>$max_visit_avg) {$max_visit_avg=$mth_item->day_avg; $max_visit_tot=$mth_item->visitors;} ?>
  
  <style type='text/css'>#atb_sum{float:left;padding:.5em 2em;margin-bottom:1em}#atb_sum .month{display:inline-block;height:3em;padding:0 .5em;cursor:pointer;text-align:center}#atb_sum .month .chart{width:100%;overflow:hidden;background:#0073AA;background:-webkit-linear-gradient(#0073AA,#FFF);background:-o-linear-gradient(#0073AA,#FFF);background:-moz-linear-gradient(#0073AA,#FFF);background:linear-gradient(#0073AA,#FFF)}#atb_sum .month .tot{font-size:.9em;color:#333}#atb_sum .month .days{display:none}#atb_sum .pointer{display:none;width:0px;height:0px;margin:auto;border-left:15px solid transparent;border-right:15px solid transparent;border-top:15px solid #a4c6dd}#atb_days{margin-top:1em;padding:0 .5em;opacity:0;font-size:.9em;-webkit-transition:all .25s;-moz-transition:all .25s;transition:all .25s}#atb_days .month{padding:0 .1em}#atb_days .month .chart{min-width:2em;background:#b71b8a;background:-webkit-linear-gradient(#b71b8a,#FFF);background:-o-linear-gradient(#b71b8a,#FFF);background:-moz-linear-gradient(#b71b8a,#FFF);background:linear-gradient(#b71b8a,#FFF)}.month .chart:hover{filter:brightness(80%)}.month .chart:hover~.pointer{display:block}</style>
  <div id='atb_sum' class='atb_control'>
    <div>
      <span style='color:#1177aa;font-weight:bold'><span class="dashicons dashicons-chart-bar"></span> Traffic Summary</span>
      <a href='#!' style='float:right' onclick="if(confirm('Compressing the data file will retain hit count data, but granular data for user tracking and page views will be less accurate.\n\nYou may compress an unlimited number of times.')) window.location='<?php echo get_admin_url(null,'admin.php?page=admin-toolbox&compress=1');?>'; else return false;" title='Compress Data File'><span style='color:#d323a0'><? if(isset($compress_result)) echo $compress_result.' hits compressed'; ?></span>&nbsp;&nbsp;<span style='opacity:.5' class="spin dashicons dashicons-share-alt"></span></a>
    </div>
    <br>
    <div style='width:100%;border-bottom:1px solid #cfe0e8;color:#0073AA;font-size:.8em'><?php echo number_format($max_visit_tot);?></div><?php
    foreach($mth as $mth_item) { if($max_visit_avg==0) $max_visit_avg=.1; ?>
      <div class='month' onclick="atb_open_month('<?php echo $mth_item->month_id;?>');" title='<?php echo number_format($mth_item->day_avg).' visits/day';?>'>
        <div class='chart' style='height:<?php echo ($mth_item->day_avg/$max_visit_avg)*100;?>%'></div>
        <div class='cal'><?php echo $mth_item->month;?></div>
        <div class='tot'><?php echo number_format($mth_item->visitors);?></div>
        <div class='pointer' id='p<?php echo $mth_item->month_id;?>'></div>
        <div class='days' id='m<?php echo $mth_item->month_id;?>'><?php
          $max_day_visit_tot=0;
          foreach($day as $day_item) if($day_item->month==$mth_item->month && $day_item->visitors>$max_day_visit_tot) $max_day_visit_tot=$day_item->visitors;?>
          <div style='width:100%;border-bottom:1px solid #cfead7;color:#06bd3a;font-size:.8em'><?php echo number_format($max_day_visit_tot);?></div><?php
          foreach($day as $day_item) {
            if($day_item->month==$mth_item->month){
              if($max_day_visit_tot==0) $max_day_visit_tot=.1; ?>
              <div class='month' title='<?php echo number_format($day_item->visitors);?> visits' <?php if($atb_version_type=='PRO') { ?>onclick="atb_loading(); window.location.href='<?php echo get_admin_url(null,'admin.php?page=admin-toolbox');?>&phv=1&date=<?php echo $day_item->date_id;?>';"<?php } ?>>
                <div class='chart' style='height:<?php echo ($day_item->visitors/$max_day_visit_tot)*100;?>%'></div>
                <div class='cal'><?php echo $day_item->day;?></div>
              </div><?php
            }
          } ?>
        </div>
      </div><?php
      } ?>
    <div id='atb_days'></div>
    <?php if($atb_version_type=='PRO') { ?><input type='button' value='recent' id='view_hits_btn' style='float:right' class='atb_button' onclick="atb_loading(); window.location='<?php echo get_admin_url(null,'admin.php?page=admin-toolbox');?>&phv=1';"><?php } ?>
  </div>
  
  <script type='text/javascript'>
    var last_sel=false;
    function atb_open_month(month) {
      document.getElementById('atb_days').style.opacity='0';
      setTimeout(function(){
        document.getElementById('atb_days').innerHTML=document.getElementById('m'+month).innerHTML;
        document.getElementById('p'+month).style.display='block';
        document.getElementById('atb_days').style.opacity='1';
        document.getElementById('atb_days').style.filter='blur(5px)';
      },250);
      setTimeout(function(){
        document.getElementById('atb_days').style.filter='';
      },500);
      if(last_sel && last_sel!=month) document.getElementById('p'+last_sel).style.display='none';
      last_sel=month;
    }
  </script><?php
}
?>